package com.chelvc.framework.elasticsearch;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.elasticsearch.annotation.Document;
import com.chelvc.framework.elasticsearch.annotation.Identity;
import com.google.common.collect.Maps;
import lombok.AllArgsConstructor;
import lombok.NonNull;

/**
 * ES数据模型上下文
 *
 * @author Woody
 * @date 2024/10/17
 */
@AllArgsConstructor
public class ModelContext {
    /**
     * 数据模型/上下文映射表
     */
    private static final Map<Class<?>, ModelContext> MODEL_CONTEXT_MAPPING = Maps.newConcurrentMap();

    /**
     * 索引名
     */
    private final String index;

    /**
     * ID字段
     */
    private final Field identity;

    /**
     * 获取索引名
     *
     * @return 索引名
     */
    public String getIndex() {
        return this.index;
    }

    /**
     * 获取文档ID
     *
     * @param document 文档对象实例
     * @return 文档ID
     */
    public String getIdentity(Object document) {
        Object value = ObjectUtils.getValue(document, this.identity);
        return ObjectUtils.ifNull(value, String::valueOf);
    }

    /**
     * 构建ES数据模型上下文
     *
     * @param model 数据模型
     * @return 数据模型上下文实例
     */
    public static ModelContext of(@NonNull Class<?> model) {
        ModelContext context = MODEL_CONTEXT_MAPPING.get(model);
        return context != null ? context : MODEL_CONTEXT_MAPPING.computeIfAbsent(model, k -> {
            // 获取索引名
            String index = ObjectUtils.ifNull(model.getAnnotation(Document.class), Document::index);
            if (StringUtils.isEmpty(index)) {
                index = StringUtils.hump2underscore(model.getSimpleName());
            }

            // 查找ID字段
            Field primary = null, backup = null;
            Field[] fields = model.getDeclaredFields();
            if (ObjectUtils.notEmpty(fields)) {
                for (Field field : fields) {
                    if (field.isAnnotationPresent(Identity.class)) {
                        primary = field;
                        break;
                    } else if (Objects.equals(field.getName(), "id")) {
                        backup = field;
                    }
                }
            }
            Field identity = ObjectUtils.ifNull(primary, backup);
            if (identity == null) {
                throw new IllegalStateException("Document model identity field not found: " + model.getName());
            }
            return new ModelContext(index, identity);
        });
    }
}
